#!/usr/bin/env Rscript
#+ echo=FALSE

devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.12/R/core_commons.R")
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.12/R/ggplot_commons.R")
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.12/R/bio/diffex_commons.R")
require(reshape)
library(splitstackshape)

guess_species <- function(gene_id){
  an_id <-gene_id[1]
  
  if(str_detect(an_id, "ENSMUSG")){
    return("mouse")
  }else if(str_detect(an_id, "ENSDARG")){
    return("zebrafish")
  }else if(str_detect(an_id, "ENSG")){
    return("human")
  }else if(str_detect(an_id, "FBgn")){
    return("fruit_fly")
  }else{
    stop(paste("could not guess species from ", an_id))
  }
}

---------------------------------------------------------------
## load the data

#geneLists <- local(get(load("smb://fileserver.mpi-cbg.de/project-ste/rnaseq_isnm1/simone_reanalysis/results/human/diffex_genes.txt")))
geneLists <- read.delim("/home/mel/MPI-CBG_work/P6_enrichment/diffex_genes.txt") %>% select(ensembl_gene_id)
#geneLists <- select(geneLists, ensembl_gene_id)

## PANTHER10 terms + protein classes
PANTHER10_terms_more <- read.delim("http://data.pantherdb.org/PANTHER10.0/ontology/Protein_Class_7.0") #%>% select(PC00000, protein.class)
PANTHER10_terms <- select(PANTHER10_terms_more, PC00000, protein.class)

## guess mart, guess species
library('biomaRt')
mart <- biomaRt::useDataset(guess_mart(geneLists$ensembl_gene_id), mart = biomaRt::useMart("ensembl"))
species <- guess_species(geneLists$ensembl_gene_id)

## PANTHER10 terms mapped to genes
## Files can be found here: ftp://ftp.pantherdb.org/sequence_classifications/10.0/PANTHER_Sequence_Classification_files/
species_url <- sprintf("ftp://ftp.pantherdb.org/sequence_classifications/10.0/PANTHER_Sequence_Classification_files/PTHR10.0_%s", species)
PANTHER10_terms_to_genes <- read.delim(species_url, header = F)

## Protein_class_relationship for constructing trees - NOT USED
#PANTHER10_protein_class_relationship <- read.delim("http://data.pantherdb.org/PANTHER10.0/ontology/Protein_class_relationship")


## Data treatment

## remove not needed columns from data frame, unite columns, delete rows with no term information
PANTHER10_terms_to_genes <- within(PANTHER10_terms_to_genes, rm(V2, V4, V5)) %>% unite(united,V6,V7,V8,V9,V10, sep=";")
PANTHER10_terms_to_genes <- PANTHER10_terms_to_genes[-grep("\\;;;;",PANTHER10_terms_to_genes$united),]
myPattern <- "PC[0-9]{5}"
PANTHER10_terms_to_genes$PCnum <- str_extract_all(PANTHER10_terms_to_genes$united, myPattern)
PANTHER10_terms_to_genes$UniProtKB <- str_extract(PANTHER10_terms_to_genes$V1, "......$")

## mapping EnsembleIDs to UniProtIDs and unnest
annotation <- getBM(attributes=c("uniprot_sptrembl", "ensembl_gene_id"), mart=mart) #, filters="uniprot_sptrembl", values=as.vector(PANTHER10_terms_to_genes$UniProtKB))
PANTHER10_terms_to_genes <- left_join(PANTHER10_terms_to_genes, annotation, by =c("UniProtKB"="uniprot_sptrembl"))

annotation2 <- getBM(attributes=c("uniprot_swissprot", "ensembl_gene_id"),  mart=mart) #, filters="uniprot_swissprot", values=as.vector(PANTHER10_terms_to_genes$UniProtKB))
PANTHER10_terms_to_genes <- left_join(PANTHER10_terms_to_genes, annotation2, by = c("UniProtKB"="uniprot_swissprot")) %>% 
  unnest() %>% select(PCnum, ensembl_gene_id)



## checking an ID
#PANTHER10_terms_to_genes %>% filter(uniprot_sptrembl=='P62805')
#annotation %>%  filter(uniprot_sptrembl=='P62805')

##Test-grep
#grep("PC[0-9]", PANTHER10_terms_to_genes$united, value = TRUE)

#PANTHER10_terms_to_genes[PANTHER10_terms_to_genes==""] <- NA 
#na.exclude(PANTHER10_terms_to_genes)
#PANTHER10_terms_to_genes <- unite(PANTHER10_terms_to_genes,united,V6,V7,V8,V9,V10, sep=";")
#PANTHER10_terms_to_genes <- cSplit(PANTHER10_terms_to_genes, splitCols = "united", sep = ";", drop = TRUE)


########################################################################################################################
#' ## Enrichment Analysis

#' This analysis was performed using [Panther](http://). The following ontologies were tested: `r paste(PANTHER10_terms, collapse=', ')`

listLabels <- geneLists %>% select(-ensembl_gene_id) %>% distinct
listLabels %<>% transform(list_label=do.call(paste, c(listLabels, sep="__")))

geneLists %>% inner_join(listLabels) %>%
    ggplot(aes(list_label)) +
    geom_bar() +
    coord_flip() +
    ggtitle("gene list sizes to be tested for term enrichment") +
    ylab("")

pantherResults <- enricher(gene = geneLists$ensembl_gene_id, 
                           organism = species, 
                           pvalueCutoff = pCutoff, 
                           readable = TRUE, 
                           TERM2GENE = PANTHER10_terms_to_genes, 
                           TERM2NAME = PANTHER10_terms) %>% summary()

enrResults <- rbind_list(mutate(pantherResults, ontology="panther"))
