# todo define project name
export project=<<PROJECTNAME>>
# screen -R $project


## madmax
if [ -n "$LSF_SERVERDIR" ]; then
    export baseDir="/projects/bioinfo/holger/projects/${project}"
    export PROJECT_SCRIPTS="/projects/bioinfo/holger/scripts/${project}"
    export NGS_TOOLS="/projects/bioinfo/scripts/ngs_tools/v1.1"
fi

## bioinfo
if [ $(hostname) == "bioinformatics-srv1" ]; then
    #<<<todo define paths on bioinfo>>>
    #export baseDir=/home/brandl/mnt/chip-seq_study/ChIPSeq_March_2015/data
    #export PROJECT_SCRIPTS=/home/brandl/mnt/chip-seq_study/ChIPSeq_March_2015/scripts
    export NGS_TOOLS="/home/brandl/mnt/mack/bioinfo/scripts/ngs_tools/v1.1"
fi


source ${NGS_TOOLS}/dge_workflow/dge_utils.sh
export PATH=${NGS_TOOLS}/dge_workflow:$PATH


## todo define igenome to be used
## igenome=/projects/bioinfo/igenomes/Canis_familiaris/Ensembl/CanFam3.1
igenome=<<<<TBD>>>>

########################################################################################################################
### Fetch the data

mcdir $baseDir/originals

wget -nc --user="USER" --password="PW"  -r --no-directories  --no-check-certificate -A "*fastq.gz" https:/projects.biotec.tu-dresden.de/ngs-filesharing/martaf/

mailme "$project: fastq download done"

### Basic QC
dge_fastqc $(ls *fastq.gz) &

## todo make sure to also copy the sample sheet in here

########################################################################################################################
### Apply renaming and merge lane replicates (but keep technical ones)

## todo adjust renaming scheme to project specifics

mcdir $baseDir/lanereps_pooled

echo '
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.10/R/core_commons.R")

options(java.parameters = "-Xmx4g" ); library(xlsx)

sheetFile <- "../originals/natalied-FC_SN678_338-2015-5-12.xls"

# nc: no culture
# na: no hormone
# ECD: ecdysone
# INS: insulin
## first number : biological repliciate
## last number: time

renaming_scheme=c("NC" = "no_culture", "NA" = "no_hormone", "ECD"="ecdysone_", "INS"="insulin_", "9"="9h", "4"="4h")

sampleSheet <- read.xlsx2(sheetFile, "Fastqfiles") %>%
    select(File, SampleName) %>%
    mutate(
        bio_replicate=str_match(SampleName, "(.).*")[,2],
        sample = str_replace(SampleName, "[0-9]*", "") %>% str_replace_all(renaming_scheme),
        bio_sample=paste(sample, bio_replicate, sep="_")
    )

write.delim(sampleSheet, file="renaming_scheme.txt")

#sampleSheet %>% count(bio_sample)

#require(ggplot2)
#ggplot(sampleSheet, aes(bio_sample)) + geom_bar() + coord_flip()
## merge lane replication


## rather write file
sampleSheet %>% group_by(bio_sample) %>% summarise(
    zcat=paste("zcat", paste(paste0("../originals/", File), collapse=" "), "| gzip -c >", paste0(bio_sample[1], ".fastq.gz"))
) %>% with(zcat) %>% write.delim(header=F, file="lane_merge.cmd", quote=F)
' | R --vanilla -q

cat lane_merge.cmd | while read line; do
    mysub "${project}__repmerge" "$line" | joblist .repmerge
done


wait4jobs .repmerge
mailme "$project: replicate merging done"

dge_fastqc $(ls *fastq.gz) &


########################################################################################################################
### Alignment the reads

mcdir $baseDir/alignments

star_align.sh $igenome $(ls $baseDir/lanereps_pooled/*.fastq.gz)


########################################################################################################################
### Differential Expression Analysis


mcdir $baseDir/dge_analysis

rend.R -e $NGS_TOOLS/dge_workflow/featcounts_deseq.R ../alignments/star_counts_matrix.txt  2>&1  | tee featcounts_deseq.log


## or use just some contrasts of interest
#echo "
#sample_1, sample_2
#unpolarised,liver_polar_stage3
#" | csvformat -T  > contrasts.txt
#
#rend.R -e $NGS_TOOLS/dge_workflow/featcounts_deseq.R --contrasts contrasts.txt ../alignments/star_counts_matrix.txt  2>&1  | tee featcounts_deseq.log


## Term enrichment
mcdir $baseDir/dge_analysis/dge_enrichment_analysis
$NGS_TOOLS/common/cp_enrichment.R --overlay_expr_data ../plot_score_matrix.txt ../degs_by_contrast.txt contrast
rend.R -e $NGS_TOOLS/common/cp_enrichment.R --overlay_expr_data ../plot_score_matrix.txt ../degs_by_contrast.txt contrast


########################################################################################################################
### Sync back to project space

## bidirectional sync with project space
## todo define mount path on bioinfo for bidirectional synching
## ~/bin/unison $baseDir ssh://bioinfo///home/brandl/mnt/<<MOUNT_PATH>> -fastcheck true -times -perms 0

# or use a uni-directional sync
#rsync -avsn --delete  $baseDir brandl@fileserver:/projects//file/server/path
mailme "$project: sync done"