#!/usr/bin/env Rscript
#+ echo=FALSE, message=FALSE

suppressMessages(library(docopt))

# retrieve and parse the command-line arguments
doc <- '
Create a small summary alignment summary files created when running star.
Usage: star_qc.R <base_directory>
'

#print(paste("args are:", commandArgs(TRUE)))
opts <- docopt(doc, commandArgs(TRUE))
#opts <- docopt(doc, ".")

if(!exists("opts")){ stop(doc); return }
#print(opts)

base_directory <- normalizePath(opts$base_directory)

if(is.na(file.info(base_directory)$isdir)){
    stop(paste("base directory", base_directory, "does not exist"))
}

devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.10/R/core_commons.R")
devtools::source_url("https://raw.githubusercontent.com/holgerbrandl/datautils/v1.10/R/ggplot_commons.R")
require.auto(knitr)

########################################################################################################################
#' # STAR Mapping Summary

#' Data directory: `r base_directory`

renaming_scheme=c("%" = "Perc", "/" = " by ", " "="_")


read_star_summary <- function(summaryFile){
    #DEBUG summaryFile="control_4.Log.final.out"

   read.delim(summaryFile, sep="|", header=F) %>%
        mutate_each(funs(str_trim)) %>%
        set_names("variable", "value") %>%
        ## remove dates
        filter(!str_detect(value, " ")) %>% 
        mutate(value=value %>% str_replace(fixed("%"), "") %>% as.numeric()) %>%
        mutate(variable=str_replace_all(variable, renaming_scheme)) %>%
        filter(!is.na(value)) %>%
        mutate(sample=basename(summaryFile) %>% trim_ext(".Log.final.out"))
}
#read_star_summary("control_4.Log.final.out")



algnSummaryLong <- ldply(list.files(base_directory, "final.out", full.names=TRUE, recursive=T), read_star_summary)

algnSummary <- algnSummaryLong %>% dcast(sample ~ variable)
algnSummary %<>% mutate(mapping_efficiency=Perc_of_reads_mapped_to_multiple_loci + Uniquely_mapped_reads_Perc)



write.delim(algnSummary, file="star_mapping_stats.txt")
# algnSummary <- read.delim("star_mapping_stats.txt")
#'  [STAR Mapping Statistics](star_mapping_stats.txt)

#scale_fill_discrete <- function (...){ scale_color_brewer(..., type = "seq", palette="Set1", "fill", na.value = "grey50") }


#+ fig.height=nrow(algnSummary)/2.5+2, fig.width=12
ggplot(algnSummary, aes(sample, Number_of_input_reads)) +
    geom_bar(stat="identity") +
    coord_flip() +
    ggtitle("read counts") +scale_y_continuous(labels=comma)


algnSummaryLong %>% filter(str_detect(variable, "Uniquely_mapped_reads_Perc|Perc_of_reads_mapped_to_multiple_loci")) %>%
    ggplot(aes(sample, value, fill=variable)) +
    geom_bar(stat="identity") +
    coord_flip()   +
    ylim(0,100) +
    ggtitle("mapping efficiency") +
    theme(legend.position='bottom')
#    guides(fill = guide_legend(position="left"))

#    guide_legend(position="left")

#plotData <- algnSummaryLong %>% filter(str_detect(variable, "Uniquely_mapped_reads_Perc|Perc_of_reads_mapped_to_multiple_loci"))
#    ggplot(plotData, aes(sample, value, color=ac(variable))) +
#    geom_bar(stat="identity", position="dodge") +
#    coord_flip()  +
#    ylim(0,100) +
#    ggtitle("mapping efficiency") +
#    guides(fill = guide_legend(position="bottom"))
#
#
#ggplot(algnSummary, aes(sample, mapping_efficiency)) +
#    geom_bar(stat="identity") +
#    coord_flip() +
#    ylim(0,100) +
#    ggtitle("mapping efficiency")
#
#ggplot(algnSummary, aes(sample, Uniquely_mapped_reads_Perc)) +
#    geom_bar(stat="identity") +
#    coord_flip() +
#    ylim(0,100) +
#    ggtitle("Unique mapper proportions")
#
#ggplot(algnSummary, aes(sample, Perc_of_reads_mapped_to_multiple_loci)) +
#    geom_bar(stat="identity") +
#    coord_flip() +
#    ylim(0,100) +
#    ggtitle("multi mapper proportions")


#ggplot(algnSummary, aes(condition, mapped_reads)) +
#    geom_bar(stat="identity") + coord_flip() +
#    ggtitle("alignments counts") +
#    scale_y_continuous(labels=comma)

########################################################################################################################
#' ## Alignment Correlation

#' Without using any transcriptome as reference, the genome can be binned and alignment counts per bin can be used to perform a correlation analysis.
#' Created using [deepTools](https://github.com/fidelram/deepTools)

## http://stackoverflow.com/questions/291813/recommended-way-to-embed-pdf-in-html
#+ results="asis"
cat("<embed src=\"./bc.pdf\" width=\"1100px\" height=\"1000px\">")